/* -*- mode: c; c-basic-offset: 2; -*- */
/******** <lalVerbatim file="ExtTrigInitSearchCV"> ********
Author: Rahkola, R
$Id: ExtTrigInitSearch.c,v 1.13 2003/07/03 18:30:56 jolien Exp $
********* </lalVerbatim> ********/

/* Include input/output functions if we're in the debug stage */
#ifdef DEBUG_EXTTRIG
#include <stdio.h>
#endif

/* standard C functions */
#include <math.h>

/* standard LAL functions*/
#include <lal/LALStdlib.h>
#include <lal/SeqFactories.h>
#include <lal/Units.h>

/* Associated headers */
#include <ExtTrig.h>
#include <ExtTrigInitSearch.h>

#ifdef __cplusplus
extern "C" {
#endif

NRCSID( EXTTRIGINITSEARCHC, "$Id: ExtTrigInitSearch.c,v 1.13 2003/07/03 18:30:56 jolien Exp $");

/************************* wrapperAPI function definitions  ******************/

/*
 * Function LALInitSearch
 *
 * Initialise the search structures
 */
void LALInitSearch(
    LALStatus             *status,
    void                 **searchParams,
    LALInitSearchParams   *initSearchParams)
{
  /* Temporary values for the various arguments which could be modified by
   * initMask settings
   */
  UINT4                   argnum, bitnum; /* to match against argc & initMask*/
  UINT4                   argeight;
  REAL4                   argthree, argfour, argnineteen, argtwenty;
  UINT4                   initMask;       /* used to store initMask param    */
  /* allocate memory to const object so we can check our type casts */
  ETSearchParams   *params = NULL;
  ETInitParams     *initParams = NULL;

  /* initialize the lal status structure */
  INITSTATUS( status, "LALInitSearch", EXTTRIGINITSEARCHC );
  ATTATCHSTATUSPTR (status);

#ifdef DEBUG_EXTTRIG
  fprintf( stdout, "exttrig:LALInitSearch\n" );
  fflush ( stdout );
#endif
  
  /*
   *
   * Sanity checks on the input, output, params
   */
  
  /* these structures should always be correctly created by the wrapperAPI */
  /* so we can use the ASSERT() macro which is disabled in the production  */
  /* code (when lal is compiled with --disable-debug)                      */
  ASSERT(     searchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  ASSERT(  ! *searchParams, status, INITSEARCHH_ENNUL, INITSEARCHH_MSGENNUL );
  ASSERT( initSearchParams, status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );

  /* the command line parameters can easily be entered incorrectly at      */
  /* runtime so we should use if() ABORT() test to ensure that errors      */
  /* are always caught even in production code                             */

  if ( !(initSearchParams->argv) ){ /* no filter parameters */
    ABORT( status, INITSEARCHH_ENULL, INITSEARCHH_MSGENULL );
  }
  if ( initSearchParams->argc < 18) { /* wrong # of filter parameters */
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS ); 
  }
 
  if (strcmp( initSearchParams->argv[0], "-filterparams" )){ /* always first */
    ABORT( status, INITSEARCHH_EARGS, INITSEARCHH_MSGEARGS );
  }
  if ( atoi( initSearchParams->argv[1] ) <= 650000000 ){ /* not GPS seconds */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }
  if ( atoi( initSearchParams->argv[2] ) < 0 ){ /* not GPS nanoseconds */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }
  if ( atof( initSearchParams->argv[13] ) >= 
       atof( initSearchParams->argv[14] ) ) { /* min freq not less than max */
    ABORT( status, INITSEARCHH_ENUMH, INITSEARCHH_MSGENUMH );
  }
  if ( ( atof( initSearchParams->argv[13] ) < 0.0 ) ||
       ( atof( initSearchParams->argv[14] ) < 0.0 ) ){ /* negative frequency */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }
  if ( atoi( initSearchParams->argv[15] ) <= 0 ){ /* incorrect sample rate */
    ABORT( status, INITSEARCHH_ENUML, INITSEARCHH_MSGENUML );
  }

  /*
   *
   * Sanity checks for initMask settings
   * "Tuning" mode (initMask & 1)
   *
   */
  initMask = atoi( initSearchParams->argv[18] );

  if (initMask & 1) {
    /* off-source c.c. spectra should not be output --
     * This is because we expect ~10^4 cross-correlations per second
     */
    if (initMask & 32) {
      ABORT( status, INITSEARCHH_EMASK, INITSEARCHH_MSGEMASK );
    }


  /*
   *
   * "Production mode (!initMask & 1)
   *
   */
  } else {

  }

  /* this is the structure that carries memory information between the     */
  /* four functions in the shared object library and between sucessive     */
  /* calls to LALApplySearch()                                             */
  *searchParams = params = LALMalloc( sizeof(ETSearchParams) );
  if ( !params )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }

  /* what follows are the same allocation and checks for the associated    */
  /* substructures -- ETInitSearchParams, ETResults (& substructures),     */
  /* and SZeroPadAndFFTParameters                                          */
  /* :NOTE: ETDataSegmentVector gets "created" last                        */
  initParams = params->initParams = 
    (ETInitParams *) LALMalloc ( sizeof(ETInitParams) );
  if ( !(params->initParams) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }
  params->result = (ETResult *) LALMalloc ( sizeof(ETResult) );
  if ( !(params->result) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }
  /* Below is commented out because the Stochastic FFT is not used anymore */
/*    params->fftParams = (SZeroPadAndFFTParameters *)  */
/*      LALMalloc ( sizeof(SZeroPadAndFFTParameters) ); */
/*    if ( !(params->fftParams) ) */
/*      { */
/*        ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC ); */
/*      } */
  params->info = (SearchSummaryInfo *) LALMalloc ( sizeof(SearchSummaryInfo) );
  if ( !(params->info) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }
  params->info->dsoName = (CHAR *) LALCalloc( dbNameLimit, sizeof(CHAR) );
  if ( !(params->info->dsoName) )
    {
      ABORT ( status, INITSEARCHH_EALOC, INITSEARCHH_MSGEALOC );
    }


  /**********************
   **********************
   *** Read in and assign the filterparams
   **********************
   **********************/
  /* assign default values for optional parameters */
  initParams->segMinDelay = 0;
  initParams->segMaxDelay = 245;

  /* base ETSearchParams parameters */
  params->searchMaster = initSearchParams->nodeClass;
  params->rank         = 0;
  params->numSlaves    = 0;
  params->curSlaves    = 0;
  strcpy(params->chan1, initSearchParams->argv[5]);
  strcpy(params->chan2, initSearchParams->argv[6]);
  strcpy(params->kernel, initSearchParams->argv[13]);
  strcpy(params->notch, initSearchParams->argv[17]);
  params->initialised  = 0;
  strcpy(params->id,"");

  /* initParams- initial search parameters */
  initParams->grbStart.gpsSeconds 
    = (INT4) atoi( initSearchParams->argv[1] );
  initParams->grbStart.gpsNanoSeconds 
    = (INT4) atoi( initSearchParams->argv[2] );
  initParams->sourceDir.longitude
    = (REAL8) atof( initSearchParams->argv[3] );
  initParams->sourceDir.latitude
    = (REAL8) atof( initSearchParams->argv[4] );
  initParams->sourceDir.system
    = COORDINATESYSTEM_EQUATORIAL;
  initParams->sampleRate
    = (UINT4) atoi( initSearchParams->argv[12] );
  initParams->segLength
    = (UINT4) (atof( initSearchParams->argv[7] ) * initParams->sampleRate);
  initParams->numOffSegments
    = (UINT4) atoi( initSearchParams->argv[8] );
  initParams->onTimeShift
    = (INT4) ( (atof( initSearchParams->argv[9] ) +
		atof( initSearchParams->argv[7] ) ) * initParams->sampleRate );
  initParams->offTimeShiftLow
    = (UINT4) ( atof( initSearchParams->argv[10] ) * initParams->sampleRate );
  initParams->offTimeShiftHigh
    = (UINT4) ( atof( initSearchParams->argv[11] ) * initParams->sampleRate );
  initParams->minFrequency
    = (REAL4) atof( initSearchParams->argv[14] );
  initParams->maxFrequency
    = (REAL4) atof( initSearchParams->argv[15] );
  strcpy(initParams->window, initSearchParams->argv[16]);
  initParams->initMask = initMask;

  /*
   * assign optional test parameters to appropriate values
   */
  if ( initParams->initMask & 256 ) {
    bitnum=9;
    for (argnum=18; argnum < initSearchParams->argc; argnum++) {
      if ( (initParams->initMask & 512) && (bitnum <= 9)) {
	initParams->segMinDelay = (UINT4) 
	  ( atof( initSearchParams->argv[argnum] ) * initParams->sampleRate );
	bitnum = 10;
      } else if ( (initParams->initMask & 1024) && (bitnum <= 10) ) {
	initParams->segMaxDelay = (UINT4)
	  ( atof( initSearchParams->argv[argnum] ) * initParams->sampleRate );
	bitnum = 11;
      } else { continue; }
    } /*  for (argnum=18; argnum < initSearchParams->argc; argnum++) ...*/
  } /*  if (initParams->initMask & (UINT4) 2^8) ...*/

#if DEBUG_EXTTRIG > 1
  /* Debug check that filterparams were assigned correctly */
  fprintf( stdout, "grbStart = %d.%09d \n",
	   initParams->grbStart.gpsSeconds,
	   initParams->grbStart.gpsNanoSeconds );
  fprintf( stdout, "sourceDir = lat: %6.4f,\tlong: %7.4f,\tsystem: %d \n",
	   initParams->sourceDir.latitude, initParams->sourceDir.longitude,
	   initParams->sourceDir.system                                    );
  fprintf( stdout, "[segLength, segMinDelay, segMaxDelay] = (in sec) [%10.8f, %10.8f, %10.8f] = (in counts) [%d, %d, %d]\n",
	   ((REAL4) initParams->segLength) / initParams->sampleRate,
	   ((REAL4) initParams->segMinDelay) / initParams->sampleRate,
	   ((REAL4) initParams->segMaxDelay) / initParams->sampleRate,
	   initParams->segLength, initParams->segMinDelay,
	   initParams->segMaxDelay                                    );
  fprintf( stdout, "numOffSegments = %d \n", initParams->numOffSegments );
  fprintf( stdout, "onTimeShift = (in sec) %10.8f = (in counts) %d\n",
	   ((REAL4) initParams->onTimeShift) / initParams->sampleRate,
	   initParams->onTimeShift                                    );
  fprintf( stdout, "[offTimeShiftLow, offTimeShiftHigh] = (in sec) [%10.8f, %10.8f] = (in counts) [%d, %d]\n",
	   ((REAL4) initParams->offTimeShiftLow) / initParams->sampleRate,
	   ((REAL4) initParams->offTimeShiftHigh) / initParams->sampleRate,
	   initParams->offTimeShiftLow, initParams->offTimeShiftHigh      );
  fprintf( stdout, "[minFrequency, maxFrequency] = [%10.3f, %10.3f]\n",
	   initParams->minFrequency, initParams->maxFrequency );
  fprintf( stdout, "initMask = %d\n", initParams->initMask );
  fflush( stdout );
#endif /*  #if DEBUG_EXTTRIG > 1*/


  /**********************
   **********************
   *** Initialize REAL4TimeSeries structures
   *** (These will be filled in LALConditionData)
   **********************
   **********************/
  params->tsChan1 = (REAL4TimeSeries *) LALCalloc(1, sizeof(REAL4TimeSeries));
  params->tsChan2 = (REAL4TimeSeries *) LALCalloc(1, sizeof(REAL4TimeSeries));


  /**********************
   **********************
   *** Initialize COMPLEX8FrequencySeries structures
   *** (These will be filled in LALConditionData)
   **********************
   **********************/
  params->fsKernel = (COMPLEX8FrequencySeries *)
    LALCalloc( 1, sizeof(COMPLEX8FrequencySeries) );
  params->fsNotch = (COMPLEX8FrequencySeries *)
    LALCalloc( 1, sizeof(COMPLEX8FrequencySeries) );


  /**********************
   **********************
   *** Initialize SZeroPadAndFFTParameters structure
   *** (not used now that stochastic FFT is not used)
   *** Initialize REALFFTPlan structure
   **********************
   **********************/
/*    params->fftParams->length = (params->initParams->segLength); */
/*  #if DEBUG_EXTTRIG > 2 */
/*    fprintf( stdout, "Creating ForwardRealFFTPlan, length %d \n", */
/*  	   params->fftParams->length                           ); */
/*    fflush( stdout ); */
/*  #endif /*  DEBUG_EXTTRIG > 2 */
  /* the following computes a measured plan for (2x - 1) datapts --     */
  /* real time-series is zero-padded from -(N-1) to -1 so that resulting*/
  /* complex frequency-series is the same length as optimal filter.     */
  params->fftParams = NULL;
#ifdef DEBUG_EXTTRIG
  LALCreateForwardRealFFTPlan( status->statusPtr,
			       &(params->fftParams),
			       initParams->segLength, 0 );
#else
  LALCreateForwardRealFFTPlan( status->statusPtr,
			       &(params->fftParams),
			       initParams->segLength, 1 );
#endif /*  DEBUG_EXTTRIG*/
  CHECKSTATUSPTR ( status );


/*    params->fftParams->window = NULL; */
/*    LALSCreateVector( status->statusPtr, &(params->fftParams->window), */
/*  		    params->fftParams->length ); */
/*    CHECKSTATUSPTR( status ); */


  /**********************
   **********************
   *** Initialize ETDataSegmentVector structure
   **********************
   **********************/
  params->data = NULL;
  LALCreateETDataSegmentVector( status->statusPtr, &(params->data),
				params->initParams);
  CHECKSTATUSPTR ( status );


  /**********************
   **********************
   *** Initialize ETSegList structure
   **********************
   **********************/
  /* create the ETSegList structure */
  params->segments = NULL;
  LALCreateETSegList( status->statusPtr, &(params->segments),
		      params->initParams);
  CHECKSTATUSPTR ( status );


  /**********************
   **********************
   *** Initialize ETResult structure
   **********************
   **********************/
  /* result- output search results */
  params->result->xCorrUnit                = lalADCCountUnit;
  /* variance1, variance2, xCorrValue, xCorrDelay, and xCorrLag will be
   * allocated on master node in LALApplySearch() 
   * xCorrSpectra will be allocated on all nodes as needed  in LALApplySearch()
   */
  params->result->xCorrValue               = NULL;
  params->result->xCorrLag                 = NULL;
  params->result->xCorrDelay               = NULL;
  params->result->xCorrSpectra             = NULL;
  params->result->badSeconds               = 0;
  params->result->length                   = 0;
  params->result->filled                   = 0;
  params->result->onSource                 = 0;


  /**********************
   **********************
   *** Initialize SearchSummaryInfo structure
   **********************
   **********************/
  /* info- search_summary database information */
  /* most info parameters will be assigned in ConditionData & ApplySearch */
  params->info->numEvents       = 0; /* will get assigned in ApplySearch  */
  params->info->numNodes        = 0; /* will get increased in ApplySearch */

  DETATCHSTATUSPTR (status);
  RETURN(status);
}

#ifdef __cplusplus
}
#endif
