/******** <lalVerbatim file="KnownPulsarDemodHV"> ********
Author: Mendell, Greg A.
$Id: KnownPulsarDemod.h,v 1.12 2003/07/03 19:38:34 jolien Exp $
********* </lalVerbatim> ********/

/* Notes: */
/*  DFT  = Discrete Fourier Transform, e.g., as computed by FFT routines. */
/*  SFT  = short time baseline DFT;  No demodulation of the data is done  */
/*  DeFT = stitched together SFTs forming demodulated longer time baseline DFT. */
/*  DeFTs remove phase modulation due to Earth's motion and that intristic to the source */
/*  Eventually DeFTs will account for amplitude modulation due to beam pattern sensitivity of the detector */

/* 09/26/01 Code first submitted to lalwrapper cvs. */
/* Revisons: */
/*  10/10/01 gam Change deltaT and tSFT to type REAL8; Check that input data interval and sample rate agrees with that requested; get units from input data. */
/*  10/10/01 gam Add error checking that data is found (unless generating test data) */
/*  10/11/01 gam Add nSFT and mCohSFTPerCall as command line filter parameters.  */
/*  10/12/01 gam Move declaration and allocation of memory into InitSearch and ConditionData; Deallocate in FinalizeSearch */
/*  10/16/01 gam Do not set up creator_db, process_id, or event_id; input filter_id, ifo, and target_name as command line filter parameters */
/*  11/05/01 gam Change filterID to dataConDesc; this is a string with data conditioning info */
/*  11/05/01 gam add dsoName and percentPad as parameters; add these and dataConDesc to dcHistory structure for each SFT */
/*  11/05/01 gam Find power in freq bin closest to f0 = pwF0 and std def pwStdDev; temporarily set SNR = (pwF0 - pwMean)/pwStdDev. */
/*  11/05/01 gam Replace pwMaxOverpwMean with pwF0OverpwMean = pwF0/pwMean. Set Confidence = 1 - exp(-pwF0/pwMean)  */
/*  11/05/01 gam Replace phaseForPWMax with phaseForPWF0 */
/*  11/05/01 gam Replace freqPWMax with freqPWF0 */
/*  11/10/01 gam Change template and test RA, Dec, Freq, Freq derivatives, and polarization from REAL4 to REAL8. */
/*  11/10/01 gam Change KNOWNPULSARDEMODH_LIGOBANDMIN to 0.0 Hz and KNOWNPULSARDEMODH_LIGOBANDMAX to 2048 Hz. */
/*  11/10/01 gam Add an SFTStats data structure; store max, mean, and stddev stats for each SFT in this structure; store values in SFT history */
/*  11/10/01 gam Add parameter nDeFT =(params->if0Max-params->if0Min)*params->mCohSFT = Number of bins in a DeFT. */
/*  11/11/01 gam Add code to handle time delays introduced into by resampling and filtering. */
/*  11/18/01 gam Allow channel as an input parameter */
/*  11/27/01 gam Add code to pad data with mean value between cutoffs using a quality channel. */
/*  12/05/01 gam Add error checking iStartTime = index to gpsStartTime is not negative or past end of input data */
/*  12/06/01 gam  Updated basic documentation */
/*  07/07/02 gam Change doDemod to type INT2 */
/*               Case 0 means compute SFTS, case 1 means compute SFTs and then do demod; case >= 2 means unpack SFTs and ephemeris data from input and do demod */
/*  07/09/02 gam Add code to unpack input SFTs */
/*  07/10/02 gam Add EphemerisData *edat = pointer to ephemeris data to DemodSearchParams */
/*  07/12/02 gam Add numNodes and numDBOutput parameters */
/*  07/12/02 gam Add flag outputDeFT: if > 0 then will output DeFT into frame file */
/*  07/18/02 gam dataConDesc is obsolete; change this to 2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
/*  08/06/02 gam Include new parameter:  demParams->sftPerCoh = params->sftPerCoh; Array with alpha index and mCohSFT, unless gaps are present. */
/*  10/15/02 gam Add normalizeSFTs parameter. Default 0 means do nothing; 1 means normalize with S_h = 2*<|SFT|> in DeFT band; 2 means calibrate first, then normalize. */
/*  02/03/03 gam If inputing SFTs, use input metadata from LDAS rather MoreTimes; add error conditions if this data is not as requested */
/*  06/26/03 gam Add INT4 Dterms parameter = terms used in the computation of the dirichlet kernel in LALDemod (Default value is 16) */

#ifndef _KNOWNPULSARDEMOD_H
#define _KNOWNPULSARDEMOD_H

#include <lal/LALRCSID.h>
#include <LALWrapperInterface.h>

NRCSID( KNOWNPULSARDEMODH, "$Id: KnownPulsarDemod.h,v 1.12 2003/07/03 19:38:34 jolien Exp $");

/* 10/10/01 gam */
/* 10/11/01 gam */
/* 12/05/01 gam */
/*  10/10/01 gam */
/*  10/11/01 gam */
/* 12/05/01 gam */

/* <lalErrTable> */
#define KNOWNPULSARDEMODH_ENULL 1
#define KNOWNPULSARDEMODH_EMOBSCOH 2
#define KNOWNPULSARDEMODH_EGPSTINT 3
#define KNOWNPULSARDEMODH_EDELTAT 4
#define KNOWNPULSARDEMODH_EINPUTN 5
#define KNOWNPULSARDEMODH_ETSFT 6
#define KNOWNPULSARDEMODH_ERA 7
#define KNOWNPULSARDEMODH_EDEC 8
#define KNOWNPULSARDEMODH_EPHASE 9
#define KNOWNPULSARDEMODH_EFREQ 10
#define KNOWNPULSARDEMODH_EFREQDERIV 11
#define KNOWNPULSARDEMODH_EGENTEST 12
#define KNOWNPULSARDEMODH_ETESTTYPE 13
#define KNOWNPULSARDEMODH_EALOC 14
#define KNOWNPULSARDEMODH_ENODATA 15
#define KNOWNPULSARDEMODH_ENDATA 16
#define KNOWNPULSARDEMODH_ETIMESTEP 17
#define KNOWNPULSARDEMODH_ESTARTTIME 18
#define KNOWNPULSARDEMODH_ESTOPTIME 19
#define KNOWNPULSARDEMODH_ENSFT 20
#define KNOWNPULSARDEMODH_ENTSFT 21
#define KNOWNPULSARDEMODH_EMCOHSFTPERCALL 22
#define KNOWNPULSARDEMODH_EDCONDESC 23
#define KNOWNPULSARDEMODH_EIFO 24
#define KNOWNPULSARDEMODH_ETARGETNAME 25
#define KNOWNPULSARDEMODH_EDATATESTTWO 26
#define KNOWNPULSARDEMODH_ECHANNEL 27
#define KNOWNPULSARDEMODH_EISTARTTIME 28
#define KNOWNPULSARDEMODH_EMISSINGSFTDATA 29
#define KNOWNPULSARDEMODH_ESTARTFREQ 30
#define KNOWNPULSARDEMODH_EFREQSTEPSIZE 31

#define KNOWNPULSARDEMODH_MSGENULL           "Null pointer"
#define KNOWNPULSARDEMODH_MSGEMOBSCOH        "mObsCoh = number of DeFTs must be 1 in current code"
#define KNOWNPULSARDEMODH_MSGEGPSTINT        "Unexpected GPS time interval"
#define KNOWNPULSARDEMODH_MSGEDELTAT         "Invalid deltaT"
#define KNOWNPULSARDEMODH_MSGEINPUTN         "Unexpected number of input data points"
#define KNOWNPULSARDEMODH_MSGETSFT           "Invalid tSFT"
#define KNOWNPULSARDEMODH_MSGERA             "Invalid right ascension"
#define KNOWNPULSARDEMODH_MSGEDEC            "Invalid declination"
#define KNOWNPULSARDEMODH_MSGEPHASE          "Invalid phase"
#define KNOWNPULSARDEMODH_MSGEFREQ           "Invalid frequency +/- 0.5*band (could be negative, outside LIGO band, or to high for sample rate)"
#define KNOWNPULSARDEMODH_MSGEFREQDERIV      "One of the frequency derivatives is possibly too large; frequency will evolve outside allowed band"
#define KNOWNPULSARDEMODH_MSGEGENTEST        "generateTestData must be y or n"
#define KNOWNPULSARDEMODH_MSGETESTTYPE       "Invalid test data type"
#define KNOWNPULSARDEMODH_MSGEALOC           "Memory allocation error"
#define KNOWNPULSARDEMODH_MSGENODATA         "No input data was found"
#define KNOWNPULSARDEMODH_MSGENDATA          "Invalid number of input data points"
#define KNOWNPULSARDEMODH_MSGETIMESTEP       "Incorrect input data time step"
#define KNOWNPULSARDEMODH_MSGESTARTTIME      "Incorrect input data start time"
#define KNOWNPULSARDEMODH_MSGESTOPTIME       "Incorrect input data stop time"
#define KNOWNPULSARDEMODH_MSGENSFT           "Invalid nSFT"
#define KNOWNPULSARDEMODH_MSGENTSFT          "nSFT and tSFT are inconsistent with deltaT"
#define KNOWNPULSARDEMODH_MSGEMCOHSFTPERCALL "Invalid mCohSFTPerCall"
#define KNOWNPULSARDEMODH_MSGEDCONDESC       "Invalid or null ifoNickName"
#define KNOWNPULSARDEMODH_MSGEIFO            "Invalid or null IFO"
#define KNOWNPULSARDEMODH_MSGETARGETNAME     "Invalid or null Target Name"
#define KNOWNPULSARDEMODH_MSGEDATATESTTWO    "Invalid input data or inputN for test type 2"
#define KNOWNPULSARDEMODH_MSGECHANNEL        "Invalid or null channel"
#define KNOWNPULSARDEMODH_MSGEISTARTTIME     "Requested GPS start time resulted in invalid index to input data."
#define KNOWNPULSARDEMODH_MSGEMISSINGSFTDATA "Some requested input SFT data is missing"
#define KNOWNPULSARDEMODH_MSGESTARTFREQ      "Input SFT start frequency does not agree with that requested"
#define KNOWNPULSARDEMODH_MSGEFREQSTEPSIZE   "Input SFT frequency step size does not agree with that expected"

/* </lalErrTable> */

/*  Define the LIGO BAND */ /* Changed 11/10/01 gam */
#define KNOWNPULSARDEMODH_LIGOBANDMIN 0.0
#define KNOWNPULSARDEMODH_LIGOBANDMAX 2048.0

/* Define cutoffs for the Gravity wave channel for handling data quality */
/* The absolute value must be above the MIN value and a below the MAX value
   to be included when computing the mean value of the input data */
#define GW_CHANNEL_MIN_VALUE_EXPECTED  0.0
#define GW_CHANNEL_MAX_VALUE_EXPECTED  1.e30

/*
 * The user-defined structure which holds data relevant to this particular
 * wrapper shared object
*/

/* 11/10/01 gam */ /* Define structure to hold statistics about a power series */
typedef struct
tagDemodPowerStats
{
	REAL8 pwMax;      /* max power */
	REAL8 freqPWMax;  /* frequency associated with pwMax */
	REAL8 pwMean;     /* mean power */
	REAL8 pwStdDev;   /* std dev of power */
}
DemodPowerStats;

typedef struct
tagDemodSearchParams
{
  /* Basic beowulf node descriptors */
  BOOLEAN searchMaster;           /* TRUE on the search master */
  UINT4   rank;                   /* Rank of this slave */
  UINT4   numSlaves;              /* Total number of slaves */
  UINT4   curSlaves;              /* Current number of slaves still working */
  UINT4   numNodes;               /*  07/12/02 gam  Should equal number of slaves + 1 for search master */
  /* CHAR    channel[dbNameLimit]; */ /* 11/18/01 gam Now an input parameter */   /* Channel Name */

  UINT4  numDBOutput; /* 07/12/02 gam Used to keep track of the number of rows written to the database */

  gpsTimeInterval times;          /* The GPS start and end time of the data */

  /* next are parameters passed as arguments */
  /* BOOLEAN doDemod; */             /* Demodulate the SFTs into DeFTs */ /* 07/07/02 gam */
  INT2    doDemod;                   /*  Case 0: compute SFTS, case 1 compute SFTs and do demod; case >= 2 input SFTs and do demod */ /* 07/07/02 gam */
  INT4    mObsCoh;                   /* Number of DeFTs to compute from input observation time */
  UINT4   gpsStartTime;              /* GPS start time of data requested */
  REAL8   deltaT;                    /* time step in seconds = 1/sample rate */ /* 10/10/01 gam */
  UINT4   inputN;                    /* number of input data points */
  REAL8   tSFT;                      /* duration in seconds of SFTs (= nSFT*deltaT = Time between start of one SFT and next SFT for sequential SFTs)*/ /* 10/10/01 gam */
  INT4    nSFT;                      /* number of time domain data points used to compute one SFTs */ /* 10/11/01 gam */
  UINT4   mCohSFTPerCall;            /* number of SFTs to compute during each call to ApplySearch */ /* 10/11/01 gam */
  REAL8   templateRA;                /* Right Ascension of template */
  REAL8   templateDec;               /* Declination of the template */
  REAL8   templatePhase0;            /* phase of template */
  REAL8   templateFreq0;             /* frequency of template */
  REAL8   templateFreq0Band;         /* Band around f0 to compute DeFT */
  REAL8   templateFreqDeriv0;        /* first deriv of freq of template */
  REAL8   templateFreqSecondDeriv0;  /* second deriv of freq of template */
  REAL8   templateFreqThirdDeriv0;   /* third deriv of freq of template */
  REAL8   templateFreqFourthDeriv0;  /* fourth deriv of freq of template */
  REAL8   templateFreqFifthDeriv0;   /* fifth deriv of freq of template */
  CHAR    generateTestData;          /* Generate test data: y = yes, n = no. */
  UINT2   testType;                  /* 0 = signal only; 1 = with noise; 2 = add test signal to input data */
  REAL8   testRA;                    /* Right Ascension of test */
  REAL8   testDec;                   /* Declination of the test */
  REAL8   testAmplitude;             /* Amplitude of the test */
  REAL8   testPhase0;                /* phase of test */
  REAL8   testFreq0;                 /* frequency of test */
  REAL8   testFreqDeriv0;            /* first deriv of freq of test */
  REAL8   testFreqSecondDeriv0;      /* second deriv of freq of test */
  REAL8   testFreqThirdDeriv0;       /* third deriv of freq of test */
  REAL8   testFreqFourthDeriv0;      /* fourth deriv of freq of test */
  REAL8   testFreqFifthDeriv0;       /* fifth deriv freq of test */
  REAL8   testPolarization;          /* Polarization of test; NOT YET IMPLEMENTED */
  INT4    testDeletions;             /* If deletions > 1 then gaps of deletions*tSFT occur between tSFT segments of test data */
  CHAR    *ifoNickName;              /* 07/18/02 gam dataConDesc is obsolete; change this to 2 character ifoNickName = H1, H2, or L1; note IFO is the site = LHO, LLO, GEO, etc.... */
  CHAR    *IFO;                      /* Identifies the interferometer name */ /* 10/16/01 gam */
  CHAR    *targetName;               /* Identifies the target of the search */ /* 10/16/01 gam */
  CHAR    channel[dbNameLimit];      /* 11/18/01 gam Now an input parameter */   /* Channel Name */
  INT4    inputSFTm;                 /* Number of input SFTs */ /* 07/09/02 gam */ /* Normally this is the same as mCohSFT, but could be smaller if gaps are present */
  INT4    inputSFTn;                 /* Number of data points in the input SFT in the input frequency band */ /* 07/09/02 gam */
  REAL8   inputSFTf0;                /* Start frequency of the input SFTs */ /* 07/09/02 gam */
  REAL8   inputSFTband;              /* Band width of the input SFTs. */ /* 07/09/02 gam */
  INT2    outputDeFT;                /* if > 0 then will output DeFT into frame file */ /* 07/12/02 gam */
  INT2    normalizeSFTs;             /* 10/15/02 gam Add normalizeSFTs parameter. Default 0 means do nothing; 1 means normalize with S_h = 2*<|SFT|> in DeFT band; 2 means calibrate first, then normalize. */
  INT4    Dterms;                    /*  06/26/03 gam Add INT4 Dterms parameter = terms used in the computation of the dirichlet kernel in LALDemod (Default value is 16) */

  /* UINT4  inputM; */  /* 10/12/01 gam */ /* number of input data segments (Computed, not input from command line) */

  /* next are containers for storing data */
  REAL4Vector* vecInputData;                 /* data received from the wrapper master */
  LALUnit     unitSFT;                       /* Unit SFTs are stored in (e.g., ADC counts per root Hertz */

  /* next are containers for storing output results */
  /*  10/12/01 gam */
  REAL8 freqPWF0;         /* freq detected with power pwF0 */ /* 11/05/01 gam */
  REAL8 pwMax;             /* power = absolute square of the amplitude at this freq */
  REAL8 phaseForPWF0;     /* phase that goes with amplitude at F0 power */ /* 11/05/01 gam */
  REAL8 pwMean;            /* mean power detected (mean computed less max power) */   /* 11/05/01 gam  (compute less pwF0) */
  REAL8 pwF0OverpwMean;   /* ratio of powMax to pwMean */ /* 11/05/01 gam */
  /* 11/05/01 gam */
  REAL8 pwF0;              /* Power in the bin closest to templateFreq0 */
  REAL8 pwStdDev;           /* Standard deviation in power (less pwF0) */

  CHAR  *dsoName;         /* Name of this DSO */ /* 11/05/01 gam */

  REAL4  percentPad;         /* Percent of input time series to produce each SFT was zero padded to fill in gaps */ /* 11/05/01 gam */

  /* parameters for keeping track of which SFTs to do, and if all SFTs are done; used with mCohSFTPerCall */
  /* 10/12/01 gam */
  UINT4 firstK;           /* Index of the first SFT to compute and store in the SFTData structure this call to ApplySearch */
  UINT4 lastK;            /* Index of the last SFT to compute and store in the SFTData Structure this call to ApplySearch */
  BOOLEAN finishedSFTs;   /* Set equal to true when all SFTS for this job have been computed */

  /* Parameters used to compute SFT */
  /* 10/12/01 gam */

  /* ComplexFFTPlan *pfwd; */  /* Plan for computing SFTs */
  RealFFTPlan  *pfwdReal;      /* Plan for computing FFTs from real input. */ /* 09/28/01 gam */

  REAL8 tCoh;      /* Duration of DeFTs.  For now, this is taken as the time input into this code */
  REAL8 dfSFT;     /* Freq resolution of SFT = 1/duration of SFTs  */
  INT4 mCohSFT;    /* Number of SFTs in one DeFT. */
  INT4 mObsSFT;    /* Total number of SFTs in the observation time. */
  INT4 nDeltaF;    /* Number of output data points in one SFT */ /* 09/28/01 gam */

  INT4 ifMin;      /* Index of minimum frequency in SFT band */
  REAL8 fMin;      /* min freq in SFT band */
  INT4 ifMax;      /* Index of maximum frequency in SFT band */
  INT4 if0Min;     /* Min search freq index in coarse-grained frequency resolution */
  REAL8 f0Min;     /* Min freq in coarse-grained freq resolution */
  INT4 if0Max;     /* Max search freq index in coarse-grained frequency resolution */
  REAL8 f0Max;     /* Max freq in coarse-grained freq resolution */

  INT4 nDeFT;     /* Number of bins in a DeFT */ /* 11/10/01 gam */

  FFT **SFTData;   /* Container for SFTs = input to LALDemod  */

  DemodPowerStats **SFTStats; /* Container for statistics about each SFT */ /* 11/10/01 gam */

  REAL4Vector *tvec;                /* Temporary holder in input when computing each SFT */ /* 09/28/01 gam */
  COMPLEX8Vector *fvec;             /* Temporary holder of output when computing each SFT */
  LIGOTimeGPS *timeStamps;          /* Container for time stamps giving initial time for which each SFT was computed. */
  gpsTimeInterval *timeIntervals;   /* Array of time intervals; needed for building frame output */

  void (*funcName)(REAL8, REAL8, REAL8, REAL8 *, REAL8 *);    /* Function that converts local t = GPS time to T = barycenter time and dT/dt) */

  EphemerisData *edat; /* 07/10/02 gam Add EphemerisData *edat = pointer to ephemeris data to DemodSearchParams */

  INT4 *sftPerCoh;  /* 08/06/02 gam */

}
DemodSearchParams;

#endif /* _KNOWNPULSARDEMOD_H */
