/**** <lalVerbatim file="ExtractSeriesCV" >
 * $Id: ExtractSeries.m4,v 1.4 2003/02/07 17:11:53 jolien Exp $
 **** </lalVerbatim> */

/**** <lalLaTeX>
 *
 * \subsection{Module \texttt{ExtractSeries.c}}
 * \label{ss:ExtractSeries.c}
 *
 * This module provides the routines used to extract time and frequency
 * series from the list of sequences of \texttt{multiDimData} contained
 * in a \texttt{LALSearchInput} structure.  Sequences are selected
 * via a regular expression pattern which is used to examine the names
 * of the sequences.  The sequence that first matches the pattern is then
 * extracted.
 *
 * \subsubsection*{Prototypes}
 * \input{ExtractSeriesCP}
 *
 * The routines \texttt{LALFindSequenceByName()} and
 * \texttt{LALFindSequenceByComment()}
 * examine the input \texttt{LALSearchInput} structure by looping over the
 * input \texttt{multiDimData} sequences.  For each sequence, the routine
 * examines the name or comment (depending on which of the two functions is
 * called) of the routine to see if it matches the specified
 * regular expression pattern.  The first sequence that so matches is then
 * extracted and a pointer to this sequence is returned.
 *
 * The routines
 * \texttt{LALExtract}$\langle\textit{type}\rangle$\texttt{TimeSeries()} and
 * \texttt{LALExtract}$\langle\textit{type}\rangle$\texttt{FrequencySeries()}
 * convert an input \texttt{multiDimData} structure into a LAL time or
 * frequency series.
 * All of the appropriate metadata contained in the \texttt{multiDimData}
 * sequence is copied into the output time or frequency series.  This includes
 * the name, the start epoch, the base frequency, the time or frequency step
 * size, and the number of points of data.  However, the units are not
 * presently copied.  They must be set manually by the user (either before
 * or after the routine is called).
 *
 * Depending on the initial condition of the output structure, the code will
 * behave in different ways.  If the time or frequency series data field
 * (which is a LAL vector) is \texttt{NULL} then the memory for the vector
 * is allocated using the appropriate LAL creation routine.  In this case,
 * the data is then \emph{copied} into the newly allocated memory.  This
 * memory must be freed when the user is finished with it (using the
 * appropriate LAL destroy vector function).
 *
 * However, if the time or frequency series data field is not \texttt{NULL},
 * but the data field of this vector \emph{is} \texttt{NULL}, then the routine
 * simply attaches the data pointer from the \texttt{multiDimData} structure
 * to the vector's data field.  Thus, there is no additional memory allocation.
 * However, this data should be treated as readonly!  The data contained in
 * the input (and thus in the time or frequency series) should not be modified.
 * Furthermore, this memory should not be freed.  This will be done
 * automatically by the \texttt{wrapperAPI}.  Using this method of accessing
 * data is somewhat advantageous in terms of memory use, but this method should
 * only be used if the data will processed non-destructively into another form
 * in new memory allocated to hold it.  E.g., if the input time series data is
 * immediately (and out-of-place) FFTed and the results put into a frequency
 * series with memory allocated to hold it, and then the time series is never
 * used again, then this method is suitable.  If this is not clear, use the
 * former method.
 *
 * If neither the data field of the time or frequency series is \texttt{NULL}
 * nor the data field of the data field of the time or frequency series
 * (i.e., the data field of the LAL vector, which is the data field of the
 * time or frequency series) is \texttt{NULL}, then the routine will return
 * an error.
 *
 * \vfill{\footnotesize\input{ExtractSeriesCV}}
 *
 **** </lalLaTeX> */

#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <ExtractSeries.h>
#include <string.h>
#include <sys/types.h>
#include <regex.h>

NRCSID( EXTRACTSERIESC, "$Id: ExtractSeries.m4,v 1.4 2003/02/07 17:11:53 jolien Exp $" );

/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALFindSequenceByName(
    LALStatus       *status,
    multiDimData   **output,
    LALSearchInput  *input,
    const CHAR      *regex
    )
{ /* </lalVerbatim> */
  int i;
  regex_t reg;

  INITSTATUS( status, "LALFindSequenceByName", EXTRACTSERIESC );

  ASSERT( regex, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( input->numberSequences, status,
      EXTRACTSERIES_EZSEQ, EXTRACTSERIES_MSGEZSEQ );
  ASSERT( input->sequences, status,
      EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( output, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );

  /* find the first sequence that matches the regex */
  *output = NULL;
  regcomp( &reg, regex, REG_NOSUB );
  for ( i = 0; i < input->numberSequences; ++i )
  {
    if ( regexec( &reg, input->sequences[i].name, 0, NULL, 0 ) )
      continue; /* this is not the sequence we're looking for */
    *output = input->sequences + i;
    break;
  }
  regfree( &reg );

  /* check to make sure that a sequence was found */
  if ( ! *output )
  {
    ABORT( status, EXTRACTSERIES_EMTCH, EXTRACTSERIES_MSGEMTCH );
  }

  RETURN( status );
}

/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALFindSequenceByComment(
    LALStatus       *status,
    multiDimData   **output,
    LALSearchInput  *input,
    const CHAR      *regex
    )
{ /* </lalVerbatim> */
  int i;
  regex_t reg;

  INITSTATUS( status, "LALFindSequenceByComment", EXTRACTSERIESC );

  ASSERT( regex, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( input->numberSequences, status,
      EXTRACTSERIES_EZSEQ, EXTRACTSERIES_MSGEZSEQ );
  ASSERT( input->sequences, status,
      EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( output, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );

  /* find the first sequence that matches the regex */
  *output = NULL;
  regcomp( &reg, regex, REG_NOSUB );
  for ( i = 0; i < input->numberSequences; ++i )
  {
    if ( regexec( &reg, input->sequences[i].comment, 0, NULL, 0 ) )
      continue; /* this is not the sequence we're looking for */
    *output = input->sequences + i;
    break;
  }
  regfree( &reg );

  /* check to make sure that a sequence was found */
  if ( ! *output )
  {
    ABORT( status, EXTRACTSERIES_EMTCH, EXTRACTSERIES_MSGEMTCH );
  }

  RETURN( status );
}

static int CheckDataType( datatype mddtype, const char *typecode )
{
  switch ( mddtype )
  {
    case boolean_lu:
      return strcmp( typecode, "BOOL" );
    case char_s:
      return strcmp( typecode, "CHAR" );
    case char_u:
      return strcmp( typecode, "UCHAR" );
    case int_2s:
      return strcmp( typecode, "I2" );
    case int_2u:
      return strcmp( typecode, "U2" );
    case int_4s:
      return strcmp( typecode, "I4" );
    case int_4u:
      return strcmp( typecode, "U4" );
    case int_8s:
      return strcmp( typecode, "I8" );
    case int_8u:
      return strcmp( typecode, "U8" );
    case real_4:
      return strcmp( typecode, "S" );
    case real_8:
      return strcmp( typecode, "D" );
    case complex_8:
      return strcmp( typecode, "C" );
    case complex_16:
      return strcmp( typecode, "Z" );
    case char_s_ptr:
      return strcmp( typecode, "BLOB" );
    case char_u_ptr:
      return strcmp( typecode, "UBLOB" );
    default:
      return 2; /* unrecognized mddtype */
  }
  return 3; /* never get here */
}
















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT2TimeSeries (
    LALStatus           *status,
    INT2TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT2TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I2" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int2s;
  }
  else
  {
    TRY( LALI2CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int2s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT2FrequencySeries (
    LALStatus                   *status,
    INT2FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT2FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I2" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int2s;
  }
  else
  {
    TRY( LALI2CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int2s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT4TimeSeries (
    LALStatus           *status,
    INT4TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT4TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I4" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int4s;
  }
  else
  {
    TRY( LALI4CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int4s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT4FrequencySeries (
    LALStatus                   *status,
    INT4FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT4FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I4" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int4s;
  }
  else
  {
    TRY( LALI4CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int4s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT8TimeSeries (
    LALStatus           *status,
    INT8TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT8TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I8" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int8s;
  }
  else
  {
    TRY( LALI8CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int8s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractINT8FrequencySeries (
    LALStatus                   *status,
    INT8FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractINT8FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "I8" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int8s;
  }
  else
  {
    TRY( LALI8CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int8s,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT2TimeSeries (
    LALStatus           *status,
    UINT2TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT2TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U2" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int2u;
  }
  else
  {
    TRY( LALU2CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int2u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT2FrequencySeries (
    LALStatus                   *status,
    UINT2FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT2FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U2" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int2u;
  }
  else
  {
    TRY( LALU2CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int2u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT4TimeSeries (
    LALStatus           *status,
    UINT4TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT4TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U4" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int4u;
  }
  else
  {
    TRY( LALU4CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int4u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT4FrequencySeries (
    LALStatus                   *status,
    UINT4FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT4FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U4" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int4u;
  }
  else
  {
    TRY( LALU4CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int4u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT8TimeSeries (
    LALStatus           *status,
    UINT8TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT8TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U8" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.int8u;
  }
  else
  {
    TRY( LALU8CreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.int8u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractUINT8FrequencySeries (
    LALStatus                   *status,
    UINT8FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractUINT8FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "U8" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.int8u;
  }
  else
  {
    TRY( LALU8CreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.int8u,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractREAL4TimeSeries (
    LALStatus           *status,
    REAL4TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractREAL4TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "S" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.real4;
  }
  else
  {
    TRY( LALSCreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.real4,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractREAL4FrequencySeries (
    LALStatus                   *status,
    REAL4FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractREAL4FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "S" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.real4;
  }
  else
  {
    TRY( LALSCreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.real4,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractREAL8TimeSeries (
    LALStatus           *status,
    REAL8TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractREAL8TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "D" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.real8;
  }
  else
  {
    TRY( LALDCreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.real8,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractREAL8FrequencySeries (
    LALStatus                   *status,
    REAL8FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractREAL8FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "D" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.real8;
  }
  else
  {
    TRY( LALDCreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.real8,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractCOMPLEX8TimeSeries (
    LALStatus           *status,
    COMPLEX8TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractCOMPLEX8TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "C" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.complex8;
  }
  else
  {
    TRY( LALCCreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.complex8,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractCOMPLEX8FrequencySeries (
    LALStatus                   *status,
    COMPLEX8FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractCOMPLEX8FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "C" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.complex8;
  }
  else
  {
    TRY( LALCCreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.complex8,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractCOMPLEX16TimeSeries (
    LALStatus           *status,
    COMPLEX16TimeSeries       *series,
    multiDimData        *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractCOMPLEX16TimeSeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
  
  if ( input->space != timeD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "Z" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dTime.startSec;
  series->epoch.gpsNanoSeconds = input->range.dTime.startNan;
  series->deltaT               = input->range.dTime.timeStepSize;
  series->f0                   = input->range.dTime.baseFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dTime.numberSamples;
    series->data->data   = input->data.complex16;
  }
  else
  {
    TRY( LALZCreateVector ( status->statusPtr, &series->data,
        input->range.dTime.numberSamples ), status );
    memcpy( series->data->data, input->data.complex16,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}















/* <lalVerbatim file="ExtractSeriesCP"> */
void
LALExtractCOMPLEX16FrequencySeries (
    LALStatus                   *status,
    COMPLEX16FrequencySeries        *series,
    multiDimData                *input
    )
{ /* </lalVerbatim> */
  INITSTATUS( status, "LALExtractCOMPLEX16FrequencySeries", EXTRACTSERIESC );
  ATTATCHSTATUSPTR( status );

  ASSERT( input, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( series, status, EXTRACTSERIES_ENULL, EXTRACTSERIES_MSGENULL );
  ASSERT( ! series->data || ! series->data->data, status,
      EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );

  if ( input->space != freqD ) /* check series type */
  {
    ABORT( status, EXTRACTSERIES_ESERT, EXTRACTSERIES_MSGESERT );
  }
  if ( CheckDataType( input->type, "Z" ) ) /* check data type */
  {
    ABORT( status, EXTRACTSERIES_ETYPE, EXTRACTSERIES_MSGETYPE );
  }

  /* copy the metadata */
  strncpy( series->name, input->name, sizeof( series->name ) );
  series->epoch.gpsSeconds     = input->range.dFreq.gpsStartTimeSec;
  series->epoch.gpsNanoSeconds = input->range.dFreq.gpsStartTimeNan;
  series->deltaF               = input->range.dFreq.freqStepSize;
  series->f0                   = input->range.dFreq.baseFreq + input->range.dFreq.startFreq;
  /* FIXME: what to do about units? */

  /*
   * Copy the data... there are several modes:
   *   1. If series->data is NULL, create it and copy data here.
   *   2. If series->data->data is NULL, just copy pointer to data.
   *   3. If series->data->data is not NULL, this is an error.
   */
  if ( series->data )
  {
    if ( series->data->data )
    {
      ABORT( status, EXTRACTSERIES_ENNUL, EXTRACTSERIES_MSGENNUL );
    }
    series->data->length = input->range.dFreq.numberSamples;
    series->data->data   = input->data.complex16;
  }
  else
  {
    TRY( LALZCreateVector ( status->statusPtr, &series->data,
        input->range.dFreq.numberSamples ), status );
    memcpy( series->data->data, input->data.complex16,
        series->data->length * sizeof( *series->data->data ) );
  }

  DETATCHSTATUSPTR( status );
  RETURN( status );
}

